@@ -5,14 +5,15 @@ from __future__ import annotations
 
 import os
 import tempfile
+from argparse import Namespace
 from itertools import zip_longest
 from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type
 
 import pytest
 
 from cobib.commands import AddCommand
-from cobib.config import config
-from cobib.database import Database
+from cobib.config import Event, config
+from cobib.database import Database, Entry
 from cobib.utils.rel_path import RelPath
 
 from .. import get_resource
@@ -44,11 +45,11 @@ class TestAddCommand(CommandTest, TUITest):
                 expected lines.
         """
         # compare with reference file
-        with open(EXAMPLE_LITERATURE, "r") as expected:
+        with open(EXAMPLE_LITERATURE, "r", encoding="utf-8") as expected:
             true_lines = expected.readlines()
-        with open(extra_filename, "r") as extra:
+        with open(extra_filename, "r", encoding="utf-8") as extra:
             true_lines += extra.readlines()
-        with open(config.database.file, "r") as file:
+        with open(config.database.file, "r", encoding="utf-8") as file:
             # we use zip_longest to ensure that we don't have more than we expect
             for line, truth in zip_longest(file, true_lines):
                 assert line == truth
@@ -131,7 +132,7 @@ class TestAddCommand(CommandTest, TUITest):
             "No input to parse. Creating new entry 'dummy' manually.",
         ) in caplog.record_tuples
 
-        with open(config.database.file, "r") as file:
+        with open(config.database.file, "r", encoding="utf-8") as file:
             lines = file.readlines()
             dummy_start = lines.index("dummy:\n")
             assert dummy_start > 0
@@ -196,6 +197,67 @@ class TestAddCommand(CommandTest, TUITest):
             except FileNotFoundError:
                 pass
 
+    @pytest.mark.parametrize(
+        ["setup"],
+        [
+            [{"git": False}],
+            [{"git": True}],
+        ],
+        indirect=["setup"],
+    )
+    def test_add_with_update(self, setup: Any) -> None:
+        """Test update option of AddCommand.
+
+        Args:
+            setup: the `tests.commands.command_test.CommandTest.setup` fixture.
+        """
+        git = setup.get("git", False)
+        AddCommand().execute(["-a", "1812.09976", "--skip-download"])
+
+        # assert initial state
+        entry = Database()["Cao2018"]
+
+        assert entry.data["author"].startswith("Yudong Cao")
+        assert entry.data["title"].startswith("Quantum Chemistry in the Age of Quantum Computing")
+        assert entry.data["arxivid"].startswith("1812.09976")
+        assert entry.data["doi"] == "10.1021/acs.chemrev.8b00803"
+        assert entry.data["primaryClass"] == "quant-ph"
+        assert entry.data["archivePrefix"] == "arXiv"
+        assert entry.data["abstract"] != ""
+        assert entry.data["year"] == 2018
+
+        assert "journal" not in entry.data.keys()
+        assert "month" not in entry.data.keys()
+        assert "number" not in entry.data.keys()
+        assert "pages" not in entry.data.keys()
+        assert "volume" not in entry.data.keys()
+
+        args = ["-d", "10.1021/acs.chemrev.8b00803", "-l", "Cao2018", "--skip-download", "--update"]
+        AddCommand().execute(args)
+
+        # assert final state
+        entry = Database()["Cao2018"]
+
+        assert entry.data["author"].startswith("Yudong Cao")
+        assert entry.data["title"].startswith("Quantum Chemistry in the Age of Quantum Computing")
+        assert entry.data["arxivid"].startswith("1812.09976")
+        assert entry.data["primaryClass"] == "quant-ph"
+        assert entry.data["archivePrefix"] == "arXiv"
+        assert entry.data["abstract"] != ""
+
+        assert entry.data["journal"] == "Chemical Reviews"
+        assert entry.data["doi"] == "10.1021/acs.chemrev.8b00803"
+        assert entry.data["month"] == "aug"
+        assert entry.data["number"] == 19
+        assert entry.data["pages"] == "10856--10915"
+        assert entry.data["volume"] == 119
+        assert entry.data["year"] == 2019
+
+        if git:
+            # assert the git commit message
+            # Note: we do not assert the arguments, because they depend on the available parsers
+            self.assert_git_commit_message("add", None)
+
     def test_skip_manual_add_if_exists(self, setup: Any, caplog: pytest.LogCaptureFixture) -> None:
         """Test manual addition is skipped if the label exists already.
 
@@ -207,7 +269,11 @@ class TestAddCommand(CommandTest, TUITest):
         assert (
             "cobib.commands.add",
             30,
-            "You tried to add a new entry 'einstein' which already exists!\n"
+            "You tried to add a new entry 'einstein' which already exists!",
+        ) in caplog.record_tuples
+        assert (
+            "cobib.commands.add",
+            30,
             "Please use `cobib edit einstein` instead!",
         ) in caplog.record_tuples
 
@@ -221,15 +287,19 @@ class TestAddCommand(CommandTest, TUITest):
             caplog: the built-in pytest fixture.
         """
         with tempfile.NamedTemporaryFile("w") as file:
-            with open(EXAMPLE_DUPLICATE_ENTRY_BIB, "r") as existing:
+            with open(EXAMPLE_DUPLICATE_ENTRY_BIB, "r", encoding="utf-8") as existing:
                 file.writelines(existing.readlines())
             file.writelines(["@article{dummy,\nauthor = {Dummy},\n}"])
             file.flush()
-            AddCommand().execute(["-b", file.name])
+            AddCommand().execute(["--skip-existing", "-b", file.name])
+        assert (
+            "cobib.commands.add",
+            30,
+            "You tried to add a new entry 'einstein' which already exists!",
+        ) in caplog.record_tuples
         assert (
             "cobib.commands.add",
             30,
-            "You tried to add a new entry 'einstein' which already exists!\n"
             "Please use `cobib edit einstein` instead!",
         ) in caplog.record_tuples
         assert (
@@ -283,6 +353,68 @@ class TestAddCommand(CommandTest, TUITest):
             # assert the git commit message
             self.assert_git_commit_message("add", None)
 
+    @pytest.mark.parametrize(
+        ["setup"],
+        [
+            [{"git": False}],
+            [{"git": True}],
+        ],
+        indirect=["setup"],
+    )
+    def test_configured_label_default(self, setup: Any) -> None:
+        """Test add command when a `label_default` is pre-configured.
+
+        Args:
+            setup: the `tests.commands.command_test.CommandTest.setup` fixture.
+        """
+        config.database.format.label_default = "{author.split()[1]}{year}"
+        git = setup.get("git", False)
+
+        AddCommand().execute(["-b", EXAMPLE_DUPLICATE_ENTRY_BIB])
+
+        assert Database()["Einstein1905"]
+
+        if git:
+            # assert the git commit message
+            self.assert_git_commit_message("add", None)
+
+    @pytest.mark.parametrize(
+        ["setup"],
+        [
+            [{"git": False}],
+            [{"git": True}],
+        ],
+        indirect=["setup"],
+    )
+    def test_disambiguate_label(self, setup: Any, caplog: pytest.LogCaptureFixture) -> None:
+        """Test label disambiguation if the provided one already exists.
+
+        Args:
+            setup: the `tests.commands.command_test.CommandTest.setup` fixture.
+            caplog: the built-in pytest fixture.
+        """
+        git = setup.get("git", False)
+
+        AddCommand().execute(["-b", EXAMPLE_DUPLICATE_ENTRY_BIB])
+
+        assert (
+            "cobib.commands.add",
+            30,
+            "You tried to add a new entry 'einstein' which already exists!",
+        ) in caplog.record_tuples
+        assert (
+            "cobib.commands.add",
+            30,
+            "The label will be disambiguated based on the configuration option: "
+            "config.database.format.label_suffix",
+        ) in caplog.record_tuples
+
+        assert Database()["einstein_a"]
+
+        if git:
+            # assert the git commit message
+            self.assert_git_commit_message("add", None)
+
     @pytest.mark.parametrize(
         ["setup"],
         [
@@ -328,3 +460,29 @@ class TestAddCommand(CommandTest, TUITest):
 
         keys = "a-b " + EXAMPLE_MULTI_FILE_ENTRY_BIB + "\n"
         self.run_tui(keys, assertion, {})
+
+    def test_event_pre_add_command(self, setup: Any) -> None:
+        """Tests the PreAddCommand event."""
+
+        @Event.PreAddCommand.subscribe
+        def hook(largs: Namespace) -> None:
+            largs.label = "dummy"
+
+        assert Event.PreAddCommand.validate()
+
+        AddCommand().execute(["-b", EXAMPLE_DUPLICATE_ENTRY_BIB])
+
+        assert "dummy" in Database().keys()
+
+    def test_event_post_add_command(self, setup: Any) -> None:
+        """Tests the PostAddCommand event."""
+
+        @Event.PostAddCommand.subscribe
+        def hook(new_entries: Dict[str, Entry]) -> None:
+            new_entries["dummy"] = new_entries.pop("einstein_a")
+
+        assert Event.PostAddCommand.validate()
+
+        AddCommand().execute(["-b", EXAMPLE_DUPLICATE_ENTRY_BIB])
+
+        assert "dummy" in Database().keys()
