@@ -4,13 +4,14 @@
 from __future__ import annotations
 
 import tempfile
+from argparse import Namespace
 from typing import TYPE_CHECKING, Any, List, Optional, Tuple, Type
 
 import pytest
 
 from cobib.commands import EditCommand
-from cobib.config import config
-from cobib.database import Database
+from cobib.config import Event, config
+from cobib.database import Database, Entry
 from cobib.utils.rel_path import RelPath
 
 from ..tui.tui_test import TUITest
@@ -50,7 +51,7 @@ class TestEditCommand(CommandTest, TUITest):
             if logs is not None:
                 assert ("cobib.commands.edit", 20, f"'{label}' was successfully edited.") in logs
 
-            with open(config.database.file, "r") as file:
+            with open(config.database.file, "r", encoding="utf-8") as file:
                 lines = file.readlines()
                 dummy_start = lines.index("dummy:\n")
                 assert dummy_start > 0
@@ -137,7 +138,9 @@ class TestEditCommand(CommandTest, TUITest):
 
             with tempfile.TemporaryDirectory() as tmpdirname:
                 path = RelPath(tmpdirname + "/einstein.pdf")
-                open(path.path, "w").close()  # pylint: disable=consider-using-with
+                open(  # pylint: disable=consider-using-with
+                    path.path, "w", encoding="utf-8"
+                ).close()
 
                 Database()["einstein"].file = str(path)
 
@@ -199,3 +202,30 @@ class TestEditCommand(CommandTest, TUITest):
             self._assert(changes=False, logs=true_log)
 
         self.run_tui("e", assertion, {})
+
+    def test_event_pre_edit_command(self, setup: Any) -> None:
+        """Tests the PreEditCommand event."""
+
+        @Event.PreEditCommand.subscribe
+        def hook(largs: Namespace) -> None:
+            largs.add = True
+            largs.label = "dummy"
+
+        assert Event.PreEditCommand.validate()
+
+        EditCommand().execute(["einstein"])
+
+        self._assert(changes=True)
+
+    def test_event_post_edit_command(self, setup: Any) -> None:
+        """Tests the PostEditCommand event."""
+
+        @Event.PostEditCommand.subscribe
+        def hook(new_entry: Entry) -> None:
+            new_entry.data["tags"] = "test"
+
+        assert Event.PostEditCommand.validate()
+
+        EditCommand().execute(["-a", "dummy"])
+
+        assert Database()["dummy"].data["tags"] == "test"
