@@ -3,12 +3,16 @@
 
 from __future__ import annotations
 
+import contextlib
 import tempfile
+from argparse import Namespace
+from io import StringIO
 from typing import TYPE_CHECKING, Any, List, Type
 
 import pytest
 
 from cobib.commands import ModifyCommand
+from cobib.config import Event
 from cobib.database import Database
 from cobib.utils.rel_path import RelPath
 
@@ -42,8 +46,15 @@ class TestModifyCommand(CommandTest, TUITest):
         ],
     )
     @pytest.mark.parametrize("add", [False, True])
+    @pytest.mark.parametrize("dry", [False, True])
     def test_command(
-        self, setup: Any, modification: str, filters: List[str], selection: bool, add: bool
+        self,
+        setup: Any,
+        modification: str,
+        filters: List[str],
+        selection: bool,
+        add: bool,
+        dry: bool,
     ) -> None:
         """Test the command itself.
 
@@ -53,6 +64,7 @@ class TestModifyCommand(CommandTest, TUITest):
             filters: the filter arguments for the command.
             selection: whether the filters are of `selection` type.
             add: whether to use append mode.
+            dry: whether to run in dry mode.
         """
         git = setup.get("git", False)
 
@@ -69,22 +81,35 @@ class TestModifyCommand(CommandTest, TUITest):
             args = ["-a"] + args
             expected = ["dummy"] + expected
 
+        if dry:
+            args.insert(0, "--dry")
+
         ModifyCommand().execute(args)
 
-        assert Database()["einstein"].data["tags"] == expected
+        if dry:
+            if add:
+                assert Database()["einstein"].data["tags"] == ["dummy"]
+            else:
+                assert "tags" not in Database()["einstein"].data.keys()
+        else:
+            assert Database()["einstein"].data["tags"] == expected
 
         if git:
-            # assert the git commit message
-            self.assert_git_commit_message(
-                "modify",
-                {
-                    "add": add,
-                    "selection": selection,
-                    "preserve_files": False,
-                    "modification": modification.split(":"),
-                    "filter": filters,
-                },
-            )
+            try:
+                # assert the git commit message
+                self.assert_git_commit_message(
+                    "modify",
+                    {
+                        "dry": False,
+                        "add": add,
+                        "selection": selection,
+                        "preserve_files": False,
+                        "modification": modification.split(":"),
+                        "filter": filters,
+                    },
+                )
+            except AssertionError:
+                assert dry
 
     @pytest.mark.parametrize(
         ["modification", "expected"],
@@ -120,7 +145,7 @@ class TestModifyCommand(CommandTest, TUITest):
             ["label:{author.split()[1]}{year}", "Einstein1905"],
             ["string:{'à' !a}", "'\\xe0'"],
             ["number:{1.2345:.2}", "1.2"],
-            ["dummy:{dummy}", "{dummy}"],
+            ["dummy:{dummy}", ""],
         ],
     )
     def test_f_string_interpretation(self, setup: Any, modification: str, expected: Any) -> None:
@@ -155,7 +180,7 @@ class TestModifyCommand(CommandTest, TUITest):
         """
         with tempfile.TemporaryDirectory() as tmpdirname:
             path = RelPath(tmpdirname + "/knuthwebsite.pdf")
-            open(path.path, "w").close()  # pylint: disable=consider-using-with
+            open(path.path, "w", encoding="utf-8").close()  # pylint: disable=consider-using-with
 
             Database()["knuthwebsite"].file = str(path)
 
@@ -267,3 +292,30 @@ class TestModifyCommand(CommandTest, TUITest):
             assert [log for log in logs if log[0] == "cobib.commands.modify"] == expected_log
 
         self.run_tui(keys, assertion, {"selection": select})
+
+    def test_event_pre_modify_command(self, setup: Any) -> None:
+        """Tests the PreModifyCommand event."""
+
+        @Event.PreModifyCommand.subscribe
+        def hook(largs: Namespace) -> None:
+            largs.modification = ("number", "2")
+
+        assert Event.PreModifyCommand.validate()
+
+        ModifyCommand().execute(["-a", "number:3", "++label", "einstein"])
+
+        assert Database()["einstein"].data["number"] == 12
+
+    def test_event_post_modify_command(self, setup: Any) -> None:
+        """Tests the PostModifyCommand event."""
+
+        @Event.PostModifyCommand.subscribe
+        def hook(labels: List[str], dry: bool) -> None:
+            print(labels)
+
+        assert Event.PostModifyCommand.validate()
+
+        with contextlib.redirect_stdout(StringIO()) as out:
+            ModifyCommand().execute(["-a", "number:3", "++label", "einstein"])
+
+            assert out.getvalue() == "['einstein']\n"
