@@ -236,7 +236,7 @@ class TestTUI(CmdLineTest, TUITest):
         config.tui.key_bindings.prompt = "p"
         TUI.bind_keys()
         assert ord(":") not in TUI.KEYDICT.keys()
-        assert ord("p") in TUI.KEYDICT.keys() and TUI.KEYDICT[ord("p")] == "Prompt"
+        assert ord("p") in TUI.KEYDICT and TUI.KEYDICT[ord("p")] == "Prompt"
         assert ("cobib.tui.tui", 20, "Binding key p to the Prompt command.") in caplog.record_tuples
 
     def test_config_unknown_command(self, caplog: pytest.LogCaptureFixture) -> None:
@@ -472,6 +472,7 @@ class TestTUI(CmdLineTest, TUITest):
         self.run_tui([SIGWINCH], assertion, {})
 
     def test_help(self, patch_curses: Any, caplog: pytest.LogCaptureFixture) -> None:
+        # pylint: disable=consider-using-f-string
         """Test `cobib.tui.tui.TUI.help`.
 
         Args:
@@ -501,7 +502,7 @@ class TestTUI(CmdLineTest, TUITest):
         ]
         inv_keys = {}
         for key, cmd in TUI.KEYDICT.items():
-            if cmd in TUI.HELP_DICT.keys():
+            if cmd in TUI.HELP_DICT:
                 inv_keys[cmd] = "ENTER" if key in (10, 13) else chr(key)
         for idx, (cmd, desc) in enumerate(TUI.HELP_DICT.items()):
             expected_log.insert(
@@ -529,6 +530,7 @@ class TestTUI(CmdLineTest, TUITest):
         """Test `cobib.tui.tui.TUI.help` while the TUI is actually running."""
 
         def assertion(screen, logs, **kwargs):  # type: ignore
+            # pylint: disable=consider-using-f-string
             header_check = ["coBib TUI Help" in line for line in screen.display]
             assert any(header_check)
             offset = header_check.index(True)
